"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var http = require("http");
var https = require("https");
var express = require("express");
var bodyParser = require("body-parser");
var cookieParser = require("cookie-parser");
var compression = require("compression");
var path = require("path");
var fs = require("fs");
var log4js = require("log4js");
var services = require("./services");
var securityShared = require("./shared/security.shared");
var logger_config_1 = require("./config/logger.config");
var promises_shared_1 = require("./shared/promises.shared");
var server_config_1 = require("./config/server.config");
var app_config_1 = require("./config/app.config");
var app = express();
var httpServer;
var httpsServer;
function init() {
    var clientPath = '../client';
    var logsPath = path.resolve(__dirname, 'logs');
    if (!fs.existsSync(logsPath))
        fs.mkdirSync(logsPath);
    log4js.configure(logger_config_1.LOGGER_CONFIG);
    var logger = log4js.getLogger('ServerScripts');
    logger.info('********************************************************');
    logger.info('* Inicializando servicio                               *');
    logger.info('********************************************************');
    app.use(log4js.connectLogger(log4js.getLogger('AccessLog'), { level: 'auto' }));
    app.use(bodyParser.urlencoded({ extended: false }));
    app.use(bodyParser.json({ limit: '5mb' }));
    app.use(bodyParser.text({ limit: '5mb' }));
    app.use(bodyParser.raw({ limit: '5mb' }));
    app.use(cookieParser());
    app.use(compression());
    app.use(function (err, req, res, next) {
        if (err && err instanceof SyntaxError && err.status >= 400 && err.status < 500 && err.message.indexOf('JSON')) {
            logger.warn('No se pudo parsear contenido JSON de request ' + req.method + ' para \'' + req.path + '\'');
            res.status(400).json({ status: false, description: 'Invalid JSON', error: 'Invalid JSON' });
        }
    });
    securityShared.InitHeaders(app);
    logger.info('Servicio ejecutando en modo \'' + server_config_1.SERVER_MODE + '\'');
    app.enable('trust proxy');
    app.set('trust proxy', true);
    app.use(function (req, res, next) {
        if (req.headers.host.slice(0, 4) !== 'www.' && req.secure) {
            next();
        }
        else if (server_config_1.SERVER_MODE === 'prod') {
            var newHost = req.headers.host.slice(0, 4) === 'www.' ? req.headers.host.slice(4) : req.headers.host;
            return res.redirect(301, 'https://' + newHost + req.originalUrl);
        }
        else {
            next();
        }
    });
    if (server_config_1.SERVER_MODE === 'prod') {
    }
    else {
    }
    app.use('/static', express.static(path.resolve(__dirname, 'static')));
    if (!app_config_1.IS_DISTRIBUTED_SERVICE) {
        app.use(securityShared.HstsGuard);
        app.disable('etag');
        services.init(app);
        app.get('/*', function (req, res, next) {
            if (req.url.match('^\/(api|auth|public|internal|popup|builder|payments|distributed|static|template|client|assets|favicon)(.*)$')) {
                res.status(404).type('txt').send('Page not found');
            }
            else {
                res.sendFile(path.resolve(__dirname, clientPath + '/index.html'));
            }
        });
    }
    else {
        app.get('/*', function (req, res, next) {
            res.status(404).type('txt').send('Page not found');
        });
    }
    return promises_shared_1.DummyPromise().then().then(function (result) {
        return new Promise(function (resolve, reject) {
            if (!server_config_1.HTTP_ENABLED)
                return resolve(true);
            logger.info('Iniciando servicio HTTP');
            console.log('Iniciando servicio HTTP');
            httpServer = http.createServer(app);
            httpServer.listen(server_config_1.HTTP_BINDING_PORT, server_config_1.HTTP_BINDING_HOST)
                .on('listening', function () {
                logger.info('Servicio HTTP escuchando en: ' + httpServer.address().address + ':' + server_config_1.HTTP_BINDING_PORT.toString());
                console.log('Servicio HTTP escuchando en: ' + httpServer.address().address + ':' + server_config_1.HTTP_BINDING_PORT.toString());
                resolve(true);
            });
        });
    }).then(function (result) {
        return new Promise(function (resolve, reject) {
            if (!server_config_1.HTTPS_ENABLED)
                return resolve(true);
            logger.info('Iniciando servicio HTTPS');
            console.log('Iniciando servicio HTTPS');
            httpsServer = https.createServer({
                key: fs.readFileSync(server_config_1.HTTPS_KEY),
                cert: fs.readFileSync(server_config_1.HTTPS_CERT)
            }, app);
            httpsServer.listen(server_config_1.HTTPS_BINDING_PORT, server_config_1.HTTP_BINDING_HOST)
                .on('listening', function () {
                logger.info('Servicio HTTPS escuchando en: ' + httpsServer.address().address + ':' + server_config_1.HTTPS_BINDING_PORT.toString());
                console.log('Servicio HTTPS escuchando en: ' + httpsServer.address().address + ':' + server_config_1.HTTPS_BINDING_PORT.toString());
                resolve(true);
            });
        });
    }).then(function (result) {
        if (!result)
            logger.error('No se pudo establecer conexión con el servicio de mail.');
        logger.info('Servicio inicializado correctamente');
        console.log('Servicio inicializado correctamente');
    }).catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || '');
        logger.info('Error al inicializar el servicio: ' + errorMsg);
        console.error('Error al inicializar el servicio: ' + errorMsg);
    });
}
exports.init = init;
