"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function AccountService(app) {
    app.get('/api/account/get_accounts/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[AccountService::get_accounts] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAccounts(search_criteria, search_expression);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'Cannot process request');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            logger.error('[AccountService::get_accounts] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_accounts', function (req, res, next) {
        var installationId = req.body.installationId || null;
        logger.info('[AccountService::get_accounts] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (installationId) {
                return mssql_service_1.MSSql.GetRelatedAccounts(installationId);
            }
            else {
                logger.info('[AccountService::get_accounts] Received params: ', installationId);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_ACCOUNT';
            else {
                if (res.headersSent)
                    return;
                res.json({ status: true, description: 'OK', data: result });
            }
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'Cannot process request');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            logger.error('[AccountService::get_accounts] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_subaccounts', function (req, res, next) {
        var installationId = req.body.installationId || null;
        logger.info('[AccountService::get_subaccounts] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (installationId) {
                return mssql_service_1.MSSql.GetRelatedSubAccounts(installationId);
            }
            else {
                logger.info('[AccountService::get_subaccounts] Received params: ', installationId);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_SUBACCOUNT';
            else {
                if (res.headersSent)
                    return;
                res.json({ status: true, description: 'OK', data: result });
            }
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'Cannot process request');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            logger.error('[AccountService::get_subaccounts] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_account_balance', function (req, res, next) {
        var platformHash = req.body.platformHash || null;
        var installationId = req.body.installationId || null;
        var accountId = req.body.account || null;
        var resData = {
            accountBalance: null,
            accountRate: null,
            subAccountsBalance: null,
            subAccountsRate: null
        };
        logger.info('[AccountService::get_account_balance] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (installationId && accountId) {
                return mssql_service_1.MSSql.GetAccountSubAccountExists(installationId, accountId, null);
            }
            else {
                logger.info('[AccountService::get_account_balance] Received params: ', installationId, accountId);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_ACCOUNT_NOT_FOUND';
            else {
                if (res.headersSent)
                    return;
                return mssql_service_1.MSSql.GetAccountSubAccountRate(installationId, accountId, null, 'CTA');
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_ACCOUNT_RATE';
            else {
                if (res.headersSent)
                    return;
                resData.accountRate = result[0];
                return mssql_service_1.MSSql.GetAccountSubAccountBalance(installationId, accountId, null, 'FREE', 'CTA');
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_ACCOUNT_BALANCE';
            else {
                if (res.headersSent)
                    return;
                resData.accountBalance = result[0];
                return mssql_service_1.MSSql.GetAccountSubAccountRate(installationId, accountId, null, 'SCTA');
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_SUBACCOUNT_RATES';
            else {
                if (res.headersSent)
                    return;
                resData.subAccountsRate = result;
                return mssql_service_1.MSSql.GetAccountSubAccountBalance(installationId, accountId, null, 'FREE', 'SCTA');
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_SUBACCOUNT_BALANCES';
            else {
                if (res.headersSent)
                    return;
                resData.subAccountsBalance = result;
                res.json({ status: true, description: 'OK', data: resData });
            }
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'Cannot process request');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            logger.error('[AccountService::get_account_balance] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AccountService = AccountService;
