"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var platform_model_1 = require("../models/platform.model");
var logger = log4js.getLogger('ServerScripts');
function VCCService(app) {
    app.post('/api/vcc/create_vcc', function (req, res, next) {
        var platformHash = req.body.platformHash || null;
        var accountId = req.body.account || null;
        var subAccountId = req.body.subAccount || null;
        var vcc = req.body.vcc || null;
        var products = req.body.products || [];
        var accountVcc = req.body.accountVcc || null;
        var platform = new platform_model_1.Platform;
        logger.info('[VCCService::create_vcc] Received ' + req.method + ' method on \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (platformHash && accountId && vcc && products && accountVcc) {
                return mssql_service_1.MSSql.GetPlatformByHash(platformHash);
            }
            else {
                logger.info('[VCCService::create_vcc] Received params: ', platformHash, accountId, subAccountId, vcc, JSON.stringify(products), JSON.stringify(accountVcc));
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_PLATFORM';
            else if (result && result.status === 0)
                throw 'SERVER_ERROR_NOT_ACTIVATED_PLATFORM';
            else {
                if (res.headersSent)
                    return;
                platform = result;
                return mssql_service_1.MSSql.GetAccountSubAccountExists(platform.installationId, accountId, subAccountId);
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_ACCOUNT_SUBACCOUNT_NOT_FOUND';
            else {
                if (res.headersSent)
                    return;
                return mssql_service_1.MSSql.GetAccountVccByAccountVcc(platform.installationId, accountId, vcc);
            }
        })
            .then(function (result) {
            if (result !== null)
                throw 'SERVER_ERROR_VCC_ALREADY_EXISTS';
            else {
                if (res.headersSent)
                    return;
                return mssql_service_1.MSSql.GetAccountVccByAccountSubAccountVcc(platform.installationId, accountId, subAccountId);
            }
        })
            .then(function (result) {
            if (result !== null)
                throw 'SERVER_ERROR_SUBACCOUNT_ALREADY_HAVE_ASSOCIATED_VCC';
            else {
                if (res.headersSent)
                    return;
                return mssql_service_1.MSSql.CreateAccountVCC(platform.installationId, accountId, subAccountId, vcc, accountVcc.transcriptionRatePerHour, accountVcc.transcriptionRatePerSecond, accountVcc.semanticAnalysisRatePerTransaction, accountVcc.cognitiveAnalysisRatePerTransaction, products.toString());
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_ACCOUNT';
            else {
                if (res.headersSent)
                    return;
                res.json({ status: true, description: 'OK', data: result });
            }
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'Cannot process request');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            logger.error('[VCCService::create_vcc] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/vcc/get_vcc_status', function (req, res, next) {
        var platformHash = req.body.platformHash || null;
        var installationId = req.body.installationId || null;
        var vcc = req.body.vcc || null;
        logger.info('[VCCService::get_vcc_status] Received ' + req.method + ' method on \'' + req.path + '\'');
        var platform = new platform_model_1.Platform;
        var resData = {
            vcc: null,
            products: null
        };
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (platformHash && vcc) {
                return mssql_service_1.MSSql.GetPlatformByHash(platformHash);
            }
            else {
                logger.info('[VCCService::get_vcc_status] Received params: ', platformHash, vcc);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_PLATFORM';
            else if (result && result.status === 0)
                throw 'SERVER_ERROR_NOT_ACTIVATED_PLATFORM';
            else {
                if (res.headersSent)
                    return;
                platform = result;
                return mssql_service_1.MSSql.GetAccountVCC(platform.installationId, vcc);
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_ACCOUNT_VCC';
            else {
                resData.vcc = result;
                return mssql_service_1.MSSql.GetAccountVccProducts(platform.installationId, vcc);
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_ACCOUNT_VCC_PRODUCTS';
            else {
                resData.products = result;
                if (res.headersSent)
                    return;
                res.json({ status: true, description: 'OK', data: resData });
            }
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'Cannot process request');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            logger.error('[PlatformService::get_vcc_status] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/vcc/get_vcc_rates', function (req, res, next) {
        var platformHash = req.body.platformHash || null;
        var installationId = req.body.installationId || null;
        var vcc = req.body.vcc || null;
        logger.info('[VCCService::get_vcc_rates] Received ' + req.method + ' method on \'' + req.path + '\'');
        var platform = new platform_model_1.Platform;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (platformHash && vcc) {
                return mssql_service_1.MSSql.GetPlatformByHash(platformHash);
            }
            else {
                logger.info('[VCCService::get_vcc_rates] Received params: ', platformHash, vcc);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_PLATFORM';
            else if (result && result.status === 0)
                throw 'SERVER_ERROR_NOT_ACTIVATED_PLATFORM';
            else {
                if (res.headersSent)
                    return;
                platform = result;
                return mssql_service_1.MSSql.GetAccountVccRates(platform.installationId, vcc);
            }
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_ACCOUNT_VCC';
            else {
                if (res.headersSent)
                    return;
                res.json({ status: true, description: 'OK', data: result });
            }
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'Cannot process request');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            logger.error('[PlatformService::get_vcc_rates] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.VCCService = VCCService;
