"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rp = require("request-promise-native");
var log4js = require("log4js");
var integration_config_1 = require("../config/integration.config");
var loggerAllegro = log4js.getLogger('Task');
function Login(user, password) {
    loggerAllegro.info('Login por API i6', JSON.stringify({ user: user, password: password }));
    var data = { user: user, password: password };
    var postOptions = {
        uri: 'http' + '://' + integration_config_1.DNS + '/inconcert/api/login/',
        method: 'POST',
        body: data,
        headers: {
            'Content-Type': 'application/json'
        },
        json: true,
        strictSSL: false
    };
    return rp(postOptions)
        .then(function (response) {
        var status = (response && response.status) ? true : false;
        var message = (response && response) ? response : '';
        return Promise.resolve({ status: status, accessToken: message.token });
    })
        .catch(function (err) {
        loggerAllegro.error('Login: Error ' + err.message);
        return Promise.resolve({ status: false, body: err.message });
    });
}
exports.Login = Login;
function SearchUsers(accessToken, request) {
    loggerAllegro.info('Request (inConcert): Listado de usuarios por API i6');
    var postOptions = {
        uri: 'http' + '://' + integration_config_1.DNS + '/inconcert/api/users/search/',
        method: 'POST',
        body: request,
        headers: {
            'Content-Type': 'application/json',
            'Authorization': accessToken
        },
        json: true,
        strictSSL: false,
        resolveWithFullResponse: false
    };
    return rp(postOptions)
        .then(function (response) {
        return Promise.resolve(response);
    })
        .catch(function (err) {
        loggerAllegro.error('SearchUsers: Error ' + err.message);
        return Promise.resolve({ status: false, body: err.message });
    });
}
exports.SearchUsers = SearchUsers;
function SaveUserByAPIi6(accessToken, user) {
    loggerAllegro.info('Request (inConcert): Creación de usuario por API i6');
    var postOptions = {
        uri: 'http' + '://' + integration_config_1.DNS + '/inconcert/api/users/save/',
        method: 'POST',
        body: user,
        headers: {
            'Content-Type': 'application/json',
            'Authorization': accessToken
        },
        json: true,
        strictSSL: false,
        resolveWithFullResponse: false
    };
    return rp(postOptions)
        .then(function (response) {
        return Promise.resolve(response);
    })
        .catch(function (err) {
        loggerAllegro.error('SaveUserByAPIi6: Error ' + err.message);
        return Promise.resolve({ status: false, body: err.message });
    });
}
exports.SaveUserByAPIi6 = SaveUserByAPIi6;
